
//**********************Voltage_source_with_MCP4921*****************************
//******************************************************************************
//******************************************************************************
//***** ATMEGA16   Frequency:8Mhz    MikroC pro for AVR v6.1.0             *****
//******************************************************************************
//**** (X):    Clear and return to decimal mode                            *****
//**** (=):    Send digital value or decimal value to  DAC chip ()         *****
//**** (+):    Increase tens in the decimal mode                           *****
//**** (-):    Decrease tens in the decimal mode                           *****
//**** ('/'):  Only Clear and continue in the same mode                    *****
//**** (on/c): Select digital mode                                         *****
//******************************************************************************
//******************************************************************************
#define Tone 150
// DAC module connections
sbit Chip_Select at PORTB0_bit;
sbit Chip_Select_Direction at DDB0_bit;
// End DAC module connections

// LCD module connections
sbit LCD_RS at PORTC2_bit;
sbit LCD_EN at PORTC3_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDC2_bit;
sbit LCD_EN_Direction at DDC3_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

// Global varibles
long Voltio=0, Vint=0, retardo=0;
int short contador=0, aux=0;
float value;
char *text;

void buzzer() {  // the buzzer sounds every time that any key is pressed
  PORTC.b1 = 1;
  delay_ms(Tone);
  PORTC.b1 = 0;
  delay_ms(Tone);
}

void DAC_Output(unsigned int valueDAC) {
  char volt;

  Chip_Select = 0;                        // Select DAC chip

  // Send High Byte
  volt = (valueDAC >> 8) & 0x0F;          // Store valueDAC[11..8] to temp[3..0]
  volt |= 0x30;                           // Define DAC setting, see MCP4921 datasheet
  SPI1_Write(volt);                       // Send high byte via SPI

  // Send Low Byte
  volt = valueDAC;                        // Store valueDAC[7..0] to temp[7..0]
  SPI1_Write(volt);                       // Send low byte via SPI

  Chip_Select = 1;                        // Deselect DAC chip
}


DIG_OR_DEC() {  // seelect digital mode or decimal mode
 buzzer();
 if(aux==3) Vint = voltio;
 else       Vint = value;
      // reduce the voltage peaks
     for(retardo=retardo; retardo<=Vint; retardo++){  //
     DAC_Output(retardo);                   // Send value to DAC chip
     Delay_ms(20);                         // Slow down key repeat pace
     }
     for(retardo=retardo; retardo>=Vint; retardo--){
     DAC_Output(retardo);                   // Send value to DAC chip
     Delay_ms(20);                         // Slow down key repeat pace
     }
}
 // the buzzer sounds every time that any key is pressed

// print values for decimal mode
void print1(){
    unsigned char ch ;
    text = "DEC";
    Lcd_Out(2,1,text);
// Write result in ASCII format
    ch = voltio/1000;
    Lcd_Chr(2,5,48+ch);
    ch = (voltio/ 100 )% 10;
    Lcd_Chr_CP(48+ch);
    Lcd_Chr(2,7,'.');
    ch = (voltio / 10 )% 10;
    Lcd_Chr(2,8,48+ch);
    ch = voltio % 10;
    Lcd_Chr_CP(48+ch);

}
// print2 values for digital mode
void print2(){
    unsigned char ch ;
    text = "DIG";
    Lcd_Out(2,1,text);
// Write result in ASCII format
    ch = voltio/1000;
    Lcd_Chr(2,5,48+ch);
    ch = (voltio/ 100 )% 10;
    Lcd_Chr_CP(48+ch);
    ch = (voltio / 10 )% 10;
    Lcd_Chr_CP(48+ch);
    ch = voltio % 10;
    Lcd_Chr_CP(48+ch);
    Lcd_Chr(2,9,' ');

}

void Voltage() {

    buzzer();
    
    if(aux==3) {
    Voltio=(10*Voltio) + contador;
    if(Voltio>4095) Voltio=contador;
    print2();
    }
    if(aux==0) {
    Voltio=(10*Voltio) + contador;
    if(Voltio>1630) Voltio=contador;
    print1();
    }
    if(aux==1) {
    Voltio=Voltio-100;
    if(Voltio<0) Voltio=0;
    print1();
    }
    if(aux==2) {
    Voltio=Voltio+100;
    if(Voltio>1630) Voltio=0;
    print1();
    }
     value=Voltio/0.398046; // resolution =1000*(vrf/4095)
  }

void set_up()
{
   DDRD = 0xF0;
   PORTD = 0x0F;
   DDC1_bit=1;
   PORTC.b1 = 0;
   Chip_Select = 1;              // Deselect DAC
   Chip_Select_Direction = 1;   // Set CS# pin as Output
   SPI1_Init();                 // Initialize SPI1 module
   Lcd_Init();                  // Initialize LCD
   Lcd_Cmd(_LCD_CLEAR);        //  Clear display
   Lcd_Cmd(_LCD_CURSOR_OFF);   //  Cursor off
}
void Main()
{
   set_up();
   text = "Voltaje_INT:";
   Lcd_Out(1,2,text);
   buzzer();
   
   while (1) //loop key check forever
   {

        // the library keypad 4x4 of Mikroc pro AVR doesn't work
        
        //first column
        PORTD = 0xEF;   //1110 1111
        Delay_ms(50);
        if (!(PIND & 0x01))  contador=7, Voltage();
        if (!(PIND & 0x02))  contador=4, Voltage();
        if (!(PIND & 0x04))  contador=1, Voltage();
        if (!(PIND & 0x08))  aux=3,      Voltage();

        //second column
        PORTD =0xDF;   //1101 1111;
        Delay_ms(50);
        if (!(PIND & 0x01))  contador=8,  Voltage();
        if (!(PIND & 0x02))  contador=5,  Voltage();
        if (!(PIND & 0x04))  contador=2,  Voltage();
        if (!(PIND & 0x08))  contador=0,  Voltage();

        //third column
        PORTD = 0xBF;   //1011 1111;
        Delay_ms(50);
        if (!(PIND & 0x01))  contador=9,  Voltage();
        if (!(PIND & 0x02))  contador=6,  Voltage();
        if (!(PIND & 0x04))  contador=3,  Voltage();
        if (!(PIND & 0x08))  DIG_OR_DEC(); //(=)

        //fourth column
        PORTD = 0x7F;    //0111 1111;
        Delay_ms(50);
        if (!(PIND & 0x01)) contador=0, Voltio=0, Voltage(); //('/')
        if (!(PIND & 0x02)) aux=0, contador=0, Voltio=0, Voltage();//(X)
        if (!(PIND & 0x04)) aux=1, contador=0, Voltage(); //(-)
        if (!(PIND & 0x08)) aux=2, contador=0, Voltage(); //(+)
   }
}